@@ -1,21 +1,29 @@
 __title__ = "tesseract-olap"
 __description__ = "A simple OLAP library."
-__version__ = "0.9.5"
+__version__ = "0.15.0"
 
 __all__ = (
+    "BackendError",
     "DataRequest",
     "DataRequestParams",
     "MembersRequest",
     "MembersRequestParams",
-    "NotAuthorized",
     "OlapServer",
+    "QueryError",
+    "SchemaError",
+    "ServerError",
     "TesseractCube",
-    "TesseractSchema",
     "TesseractError",
+    "TesseractSchema",
 )
 
-from .exceptions import TesseractError
-from .exceptions.query import NotAuthorized
+from .exceptions import (
+    BackendError,
+    QueryError,
+    SchemaError,
+    ServerError,
+    TesseractError,
+)
 from .query import DataRequest, DataRequestParams, MembersRequest, MembersRequestParams
 from .schema import TesseractCube, TesseractSchema
 from .server import OlapServer
