@@ -5,16 +5,21 @@ from .cache import (
     DummyProvider,
     LfuProvider,
 )
-from .models import Backend, ParamManager, Result, Session
+from .dataframe import JoinStep, growth_calculation, rename_columns
+from .models import Backend, ParamManager, Result, Session, chunk_queries
 
 __all__ = (
     "Backend",
     "CacheConnection",
     "CacheConnectionStatus",
     "CacheProvider",
+    "chunk_queries",
     "DummyProvider",
+    "growth_calculation",
+    "JoinStep",
     "LfuProvider",
     "ParamManager",
     "Result",
     "Session",
+    "rename_columns",
 )
