@@ -5,13 +5,13 @@ from .strings import (
     NAN_VALUES,
     TRUTHY_STRINGS,
     get_localization,
-    hide_dsn_password,
     is_numeric,
     numerify,
     shorthash,
     stringify,
 )
 from .types import AnyDict, AnyTuple, Array, Prim, T
+from .url import URL, hide_dsn_password
 
 __all__ = (
     "AnyDict",
@@ -28,4 +28,5 @@ __all__ = (
     "stringify",
     "T",
     "TRUTHY_STRINGS",
+    "URL",
 )
