@@ -10,10 +10,11 @@ __all__ = (
 class TesseractError(Exception):
     """Base class for exceptions in `tesseract_olap` module."""
 
-    code = 500
+    code: int = 500
+    message: str
 
-    def __init__(self, message: str) -> None:
-        super().__init__(message)
+    def __init__(self, message: str, *args) -> None:
+        super().__init__(*args)
         self.message = message
 
 
@@ -24,7 +25,7 @@ class BackendError(TesseractError):
 class QueryError(TesseractError):
     """Base class for exceptions in `tesseract_olap.query` module."""
 
-    code = 400
+    code: int = 400
 
 
 class SchemaError(TesseractError):
