@@ -1,25 +1,20 @@
-from typing import Dict, List, Optional, Set, Tuple, Union
+from typing import Dict, List, Optional, Set, Union
 
 from fastapi import Depends, Header, Query, Request
 from logiclayer import AuthToken, AuthTokenType
-from typing_extensions import Annotated, Literal
+from typing_extensions import Annotated
 
 from tesseract_olap.common import FALSEY_STRINGS, TRUTHY_STRINGS
 from tesseract_olap.query import (
     AnyOrder,
     DataRequest,
     DataRequestParams,
+    FilterCondition,
+    FilterIntent,
     MembersRequest,
     MembersRequestParams,
-    NumericConstraint,
 )
 
-SingleFilterCondition = Tuple[NumericConstraint]
-DoubleFilterCondition = Tuple[
-    NumericConstraint, Literal["and", "or"], NumericConstraint
-]
-FilterCondition = Union[SingleFilterCondition, DoubleFilterCondition]
-
 
 def auth_token(
     header_auth: Annotated[Optional[str], Header(alias="authorization")] = None,
@@ -29,7 +24,7 @@ def auth_token(
     if header_jwt:
         return AuthToken(AuthTokenType.JWTOKEN, header_jwt)
     if query_token:
-        return AuthToken(AuthTokenType.APIKEY, query_token)
+        return AuthToken(AuthTokenType.SEARCHPARAM, query_token)
     if header_auth:
         if header_auth.startswith("Bearer "):
             return AuthToken(AuthTokenType.JWTOKEN, header_auth[7:])
@@ -100,35 +95,9 @@ Where a single cut definition is composed by the Level name, colon, and the list
     }
 
 
-def _filter_parse_condition(value: str) -> NumericConstraint:
-    comparison, scalar = value.split(".", 1)
-    return comparison, float(scalar)
-
-
-def _filter_parse(
-    value: str,
-) -> Tuple[str, Union[SingleFilterCondition, DoubleFilterCondition]]:
-    field, condition = value.split(".", 1)
-    if ".and." in condition:
-        cond1, cond2 = condition.split(".and.")
-        return field, (
-            _filter_parse_condition(cond1),
-            "and",
-            _filter_parse_condition(cond2),
-        )
-    if ".or." in condition:
-        cond1, cond2 = condition.split(".or.")
-        return field, (
-            _filter_parse_condition(cond1),
-            "or",
-            _filter_parse_condition(cond2),
-        )
-    return field, (_filter_parse_condition(condition),)
-
-
 def query_filters(
     filters: Annotated[
-        str,
+        List[str],
         Query(
             description="""\
 Specifies filters over the values of the calculated aggregations.
@@ -144,10 +113,15 @@ Where
     `{joint}` : `Literal["and", "or"]`, the logic operation to apply between two conditions;
 """
         ),
-    ] = "",
-):
+    ] = [],
+) -> Dict[str, FilterCondition]:
     """FastAPI Dependency to parse filter parameters."""
-    return dict(_filter_parse(item) for item in filters.split(",") if item != "")
+    return dict(
+        FilterIntent.model_validate(item).as_tuple()
+        for filter in filters
+        for item in filter.split(",")
+        if item != ""
+    )
 
 
 def query_parents(
@@ -243,18 +217,21 @@ def dataquery_params(
             description="A list of the measure names to retrieve from the aggregated data."
         ),
     ],
+    aliases: Optional[str] = None,
     cuts_exclude: Dict[str, List[str]] = Depends(query_cuts_exclude),
     cuts_include: Dict[str, List[str]] = Depends(query_cuts_include),
     filters: Dict[str, FilterCondition] = Depends(query_filters),
     locale: Optional[str] = None,
     pagination: Annotated[
         str, Query(alias="limit", description=DESCRIPTION_PAGINATION)
-    ] = "",
+    ] = "0",
     parents: Union[bool, List[str]] = Depends(query_parents),
     properties: Optional[str] = None,
     ranking: Union[bool, Dict[str, AnyOrder]] = Depends(query_ranking),
     sorting: Annotated[str, Query(alias="sort", description=DESCRIPTION_SORTING)] = "",
     time: Optional[str] = None,
+    top: Optional[str] = None,
+    growth: Optional[str] = None,
 ):
     """FastAPI Dependency to parse parameters into a DataRequest object."""
     params: DataRequestParams = {
@@ -278,6 +255,15 @@ def dataquery_params(
     if time is not None:
         params["time"] = time
 
+    if top is not None:
+        params["top"] = top
+
+    if growth is not None:
+        params["growth"] = growth
+
+    if aliases is not None:
+        params["aliases"] = aliases
+
     return DataRequest.new(cube_name, params)
 
 
@@ -289,7 +275,7 @@ def membersquery_params(
     locale: Optional[str] = None,
     pagination: Annotated[
         str, Query(alias="limit", description=DESCRIPTION_PAGINATION)
-    ] = "",
+    ] = "0",
     parents: bool = False,
     search: str = "",
 ):
