@@ -7,6 +7,7 @@ purpose is organize and easily obtain the data needed for later steps.
 
 import hashlib
 from collections import defaultdict
+from copy import copy
 from dataclasses import dataclass, field
 from itertools import chain
 from typing import Dict, List, Mapping, Optional, Tuple, Union
@@ -17,10 +18,12 @@ from typing_extensions import Literal
 from tesseract_olap.common import stringify
 from tesseract_olap.exceptions.query import (
     InvalidEntityName,
+    InvalidParameter,
     MissingMeasures,
     NotAuthorized,
 )
 from tesseract_olap.schema import (
+    CalculatedMeasure,
     CubeTraverser,
     DimensionTraverser,
     HierarchyTraverser,
@@ -29,14 +32,18 @@ from tesseract_olap.schema import (
     SchemaTraverser,
 )
 
-from .enums import RestrictionScale
+from .enums import TimeScale
 from .models import (
     CutIntent,
+    GrowthIntent,
     HierarchyField,
+    JoinIntent,
+    JoinOnColumns,
     LevelField,
     MeasureField,
     PaginationIntent,
     SortingIntent,
+    TopkIntent,
 )
 from .requests import DataRequest, MembersRequest
 
@@ -59,6 +66,21 @@ class DataQuery:
     options: Mapping[str, bool] = field(default_factory=dict)
     pagination: "PaginationIntent" = field(default_factory=PaginationIntent)
     sorting: Optional["SortingIntent"] = None
+    topk: Optional["TopkIntent"] = None
+    growth: Optional["GrowthIntent"] = None
+
+    def __copy__(self):
+        return DataQuery(
+            cube=self.cube,
+            locale=self.locale,
+            fields_qualitative=tuple(copy(item) for item in self.fields_qualitative),
+            fields_quantitative=tuple(copy(item) for item in self.fields_quantitative),
+            options=dict(self.options),
+            pagination=self.pagination.model_copy(),
+            sorting=None if self.sorting is None else self.sorting.model_copy(),
+            topk=None if self.topk is None else self.topk.model_copy(),
+            growth=None if self.growth is None else self.growth.model_copy(),
+        )
 
     def __repr__(self):
         gen_levels = (
@@ -77,6 +99,8 @@ class DataQuery:
             f"options={stringify(self.options)}",
             f"pagination={repr(self.pagination)}",
             f"sorting={repr(self.sorting)}",
+            f"topk={repr(self.topk)}",
+            f"growth={repr(self.growth)}",
         )
         return f"{type(self).__name__}({', '.join(params)})"
 
@@ -84,6 +108,16 @@ class DataQuery:
     def key(self) -> str:
         return hashlib.md5(repr(self).encode("utf-8")).hexdigest()
 
+    @property
+    def count_key(self) -> str:
+        gen_levels = (
+            repr(lvlfi)
+            for hiefi in sorted(self.fields_qualitative, key=lambda x: x.dimension.name)
+            for lvlfi in hiefi.levels
+        )
+        string = f"DataQuery(cube={self.cube.name!r}, fields=({', '.join(gen_levels)}))"
+        return "d" + hashlib.md5(string.encode("utf-8")).hexdigest()
+
     @property
     def columns(self) -> Dict[str, MemberType]:
         locale = self.locale
@@ -96,10 +130,10 @@ class DataQuery:
             if msrfi.with_ranking
         )
         gen_level = (
-            (name, lvlfi.level.key_type)
+            (column.alias, lvlfi.level.key_type)
             for hiefi in self.fields_qualitative
             for lvlfi in hiefi.drilldown_levels
-            for _, name, _ in lvlfi.iter_columns(locale)
+            for column in lvlfi.iter_columns(locale)
         )
         return dict(chain(gen_measure, gen_ranking, gen_level))
 
@@ -112,10 +146,14 @@ class DataQuery:
         of the :class:`InvalidQuery` error.
         """
         if not schema.is_authorized(request):
-            raise NotAuthorized(f"Cube({request.cube})", request.roles)
+            raise NotAuthorized(repr(request.cube), request.roles)
 
         cube = schema.get_cube(request.cube)
 
+        # TODO: consider replacing the Intents with Directives for pagination,
+        # sorting, topk, where field names are replaced with schema objects,
+        # or errors are raised if they don't exist
+
         return cls(
             cube=cube,
             fields_qualitative=_get_data_hierarfields(cube, request),
@@ -124,9 +162,123 @@ class DataQuery:
             options=request.options,
             pagination=request.pagination,
             sorting=request.sorting,
+            topk=request.topk,
+            growth=request.growth,
         )
 
 
+@dataclass(eq=False, order=False)
+class DataMultiQuery:
+    initial: "DataQuery"
+    join_with: Tuple[Tuple["DataQuery", "JoinIntent"], ...]
+
+    def get_annotations(self):
+        queries = chain([self.initial], (item[0] for item in self.join_with))
+        return {query.cube.name: dict(query.cube.annotations) for query in queries}
+
+    @classmethod
+    def from_requests(
+        cls,
+        schema: "SchemaTraverser",
+        requests: List["DataRequest"],
+        joins: List["JoinIntent"],
+    ):
+        """Validates the DataRequests into DataQueries and reorganizes the JoinIntent operations that are intended to be performed with each of them.
+
+        By itself this class is not intended to be sent to the backend for execution (as the join operation is performed using DataFrames), but in the future and depending on the Backend implementation, it's subject to change.
+        """
+
+        queries = [DataQuery.from_request(schema, request) for request in requests]
+
+        if len(queries) - 1 != len(joins):
+            raise InvalidParameter(
+                "joins",
+                "Invalid number of 'joins' parameters; it must be one per query intended to be joined with the initial.",
+            )
+
+        join_with = tuple(
+            cls._yield_step_pair(left, right, join)
+            for left, right, join in zip(queries[:-1], queries[1:], joins)
+        )
+
+        return cls(initial=queries[0], join_with=join_with)
+
+    @classmethod
+    def _yield_step_pair(
+        cls, query_left: DataQuery, query_right: DataQuery, join_intent: JoinIntent
+    ) -> Tuple["DataQuery", "JoinIntent"]:
+        columns_left = cls._get_columns(query_left)
+        columns_right = cls._get_columns(query_right)
+
+        left_fields = dict(columns_left)
+        right_fields = dict(columns_right)
+
+        join_on = join_intent.on
+        if isinstance(join_on, str):
+            join_on = [join_on]
+
+        if isinstance(join_on, list):
+            join_intent.on = [
+                column
+                for column in join_on
+                if left_fields.get(column) == right_fields.get(column)
+            ]
+        elif isinstance(join_on, JoinOnColumns):
+            if join_on.left_on not in left_fields:
+                raise InvalidParameter(
+                    "on.left_on",
+                    f"Column '{join_on.left_on}' is not present in the left dataset at this stage of the merge.",
+                )
+            if join_on.right_on not in right_fields:
+                raise InvalidParameter(
+                    "on.right_on",
+                    f"Column '{join_on.right_on}' is not present in the right dataset at this stage of the merge.",
+                )
+        else:
+            common_aliases = set(left_fields.keys()) & set(right_fields.keys())
+            common_fields = set(left_fields.values()) & set(right_fields.values())
+            if len(common_aliases) > 0:
+                join_intent.on = list(common_aliases)
+            elif len(common_fields) > 0:
+                fields = list(common_fields)
+                join_intent.on = JoinOnColumns(
+                    left_on=list(
+                        next(
+                            key for key, value in left_fields.items() if value == field
+                        )
+                        for field in fields
+                    ),
+                    right_on=list(
+                        next(
+                            key for key, value in right_fields.items() if value == field
+                        )
+                        for field in fields
+                    ),
+                )
+            else:
+                raise InvalidParameter(
+                    "request", "Couldn't find common columns between requested queries"
+                )
+
+        return query_right, join_intent
+
+    @classmethod
+    def _get_columns(cls, query: "DataQuery"):
+        locale = query.locale
+        fact_table = query.cube.table
+        return {
+            (alias, f"{table.name}.{column}")
+            for table, column, alias in (
+                next(
+                    (hiefi.table or fact_table, column.name, column.alias)
+                    for column in lvlfi.iter_columns(locale)
+                )
+                for hiefi in query.fields_qualitative
+                for lvlfi in hiefi.drilldown_levels
+            )
+        }
+
+
 @dataclass(eq=False, order=False, repr=False)
 class MembersQuery:
     """Internal MembersQuery class."""
@@ -154,13 +306,17 @@ class MembersQuery:
     def key(self) -> str:
         return hashlib.md5(repr(self).encode("utf-8")).hexdigest()
 
+    @property
+    def count_key(self) -> str:
+        return "m" + self.key
+
     @property
     def columns(self) -> Dict[str, MemberType]:
         locale = self.locale
         return {
-            name: lvlfi.level.key_type
+            column.alias: lvlfi.level.key_type
             for lvlfi in self.hiefield.levels
-            for _, name, _ in lvlfi.iter_columns(locale)
+            for column in lvlfi.iter_columns(locale)
         }
 
     @classmethod
@@ -195,6 +351,7 @@ def _get_data_hierarfields(cube: "CubeTraverser", req: "DataRequest"):
     drilldown_set = req.drilldowns
     property_set = req.properties
     caption_set = req.captions
+    alias_map = {**req.aliases}
     cut_map = {**req.cuts}
 
     with_parents = req.parents
@@ -210,7 +367,7 @@ def _get_data_hierarfields(cube: "CubeTraverser", req: "DataRequest"):
         granularity = time_restr.level
         time_level = (
             cube.get_time_level(granularity.value)
-            if isinstance(granularity, RestrictionScale)
+            if isinstance(granularity, TimeScale)
             else cube.get_time_level(granularity)
         )
         involved_levels.add(time_level.name)
@@ -221,11 +378,18 @@ def _get_data_hierarfields(cube: "CubeTraverser", req: "DataRequest"):
     hie_store: Mapping[HierarchyTraverser, List[LevelTraverser]] = defaultdict(list)
 
     for name in involved_levels:
-        dimension, hierarchy, level = level_map[name]
+        try:
+            dimension, hierarchy, level = level_map[name]
+        except KeyError:
+            raise InvalidParameter(
+                "drilldowns",
+                f"Could not find a Level named '{name}' in the '{cube.name}' cube.",
+            ) from None
         if dim_store.get(dimension, hierarchy) != hierarchy:
-            raise ValueError(
+            raise InvalidParameter(
+                "drilldowns",
                 "Multiple Hierarchies from the same Dimension are being requested. "
-                "Only a single Hierarchy can be used at a time for a query."
+                "Only a single Hierarchy can be used at a time for a query.",
             )
         dim_store[dimension] = hierarchy
         hie_store[hierarchy].append(level)
@@ -250,13 +414,14 @@ def _get_data_hierarfields(cube: "CubeTraverser", req: "DataRequest"):
 
         level, member = default_member
         levels.append(level)
-        cut_map[level.name] = CutIntent.new(level.name, incl=[member], excl=[])
+        cut_map[level.name] = CutIntent.model_validate((level.name, [member], []))
 
     def _compose_field(level: "LevelTraverser", is_drilldown: bool) -> "LevelField":
         """Capsules the logic to fill a LevelField instance with data from both
         a Drilldown and a Cut.
         """
         kwargs = {
+            "level_alias": alias_map.get(level.name) if is_drilldown else None,
             "is_drilldown": is_drilldown,
             "properties": frozenset(
                 prop for prop in level.properties if prop.name in property_set
@@ -327,15 +492,26 @@ def _get_data_measurefields(cube: "CubeTraverser", req: "DataRequest"):
 
     filter_constr = {item.field: item.condition for item in req.filters.values()}
 
+    calcmeasure_deps = (
+        column
+        for measure in (cube.get_measure(name) for name in req.measures)
+        if isinstance(measure, CalculatedMeasure)
+        for column in measure.dependencies
+        if column in cube.measure_map
+    )
+
     return tuple(
         MeasureField(
             measure=item,
+            is_measure=item.name in req.measures,
             with_ranking=ranking_flags.get(item.name),
             constraint=filter_constr.get(item.name),
         )
         for measure in (
             cube.get_measure(name)
-            for name in sorted(req.measures.union(filter_constr.keys()))
+            for name in sorted(
+                req.measures.union(filter_constr.keys(), calcmeasure_deps)
+            )
         )
         for item in measure.and_submeasures()
     )
