@@ -70,6 +70,9 @@ class SchemaTraverser(Mapping[str, "CubeTraverser"]):
     shared entities and their usages.
     """
 
+    schema: "Schema"
+    cube_map: OrderedDict[str, "CubeTraverser"]
+
     def __init__(self, schema: "Schema"):
         self.schema = schema
         self.cube_map = OrderedDict(
@@ -121,7 +124,7 @@ class SchemaTraverser(Mapping[str, "CubeTraverser"]):
 
         for cube in self.cube_map.values():
             cubename = cube.name
-            nameset = {item.name: type(item).__name__ for item in cube.measures}
+            nameset = {}
 
             for measure in cube.measures:
                 for item in measure.and_submeasures():
@@ -143,7 +146,6 @@ class SchemaTraverser(Mapping[str, "CubeTraverser"]):
 
             # Index fact tables
             tables[table.name].update(
-                [table.primary_key],
                 (
                     item.key_column
                     for measure in cube.measures
@@ -160,7 +162,6 @@ class SchemaTraverser(Mapping[str, "CubeTraverser"]):
 
                 # Index dimension tables
                 tables[table.name].update(
-                    [table.primary_key],
                     (
                         item
                         for level in hierarchy.levels
@@ -173,7 +174,7 @@ class SchemaTraverser(Mapping[str, "CubeTraverser"]):
                     ),
                 )
 
-        return tables
+        return dict(tables)
 
 
 class CubeTraverser:
@@ -615,10 +616,10 @@ class LevelTraverser(EntityUsageTraverser[Level, LevelUsage]):
     """Allows seamless aliasing of values between a Level and a LevelUsage."""
 
     depth: int
+    count: int
     key_column: str
     key_type: MemberType
     name_column_map: Mapping[str, str]
-    name: str
     property_map: Mapping[str, "PropertyTraverser"]
 
     def __init__(self, entity: "Level", usage: Optional["LevelUsage"] = None):
@@ -670,7 +671,7 @@ def validate_entity_names(
     cube: str, item: Union["AnyMeasure", "EntityUsageTraverser"], record: Dict[str, str]
 ):
     """Verifies the name of the Entity is valid and unique."""
-    if isinstance(item, AnyMeasure):
+    if isinstance(item, (Measure, CalculatedMeasure)):
         origin = item
     else:
         origin = item._entity if item._usage is None else item._usage
