@@ -22,8 +22,9 @@ from typing import (
 import immutables as immu
 from lxml import etree
 
-from tesseract_olap.common import is_numeric, numerify
+from tesseract_olap.common import TRUTHY_STRINGS, T, is_numeric, numerify
 from tesseract_olap.exceptions.schema import (
+    DuplicateKeyError,
     InvalidXMLAttributeValue,
     MissingXMLAttribute,
     MissingXMLNode,
@@ -52,6 +53,9 @@ XMLEntity = Union[
     "XMLCalculatedMeasure",
 ]
 
+AnyXMLDimension = TypeVar(
+    "AnyXMLDimension", bound=Union["XMLSharedDimension", "XMLPrivateDimension"]
+)
 AnyXMLEntity = TypeVar("AnyXMLEntity", bound=XMLEntity)
 
 
@@ -64,15 +68,17 @@ class XMLSchema(models.Schema):
         name = _get_attr(node, "name")
         logger.debug("Parsing node <%s name='%s' />", cls.tag, name)
 
-        cube_gen = _yield_children_nodes(node, XMLCube)
-        shareddim_gen = _yield_children_nodes(node, XMLSharedDimension)
-        sharedtbl_gen = _yield_children_nodes(node, XMLInlineTable)
-
         return cls(
             name=name,
-            cube_map=OrderedDict(cube_gen),
-            shared_dimension_map=immu.Map(shareddim_gen),
-            shared_table_map=immu.Map(sharedtbl_gen),
+            cube_map=OrderedDict(
+                _raise_if_duplicate(_yield_children_nodes(node, XMLCube))
+            ),
+            shared_dimension_map=immu.Map(
+                _raise_if_duplicate(_yield_children_nodes(node, XMLSharedDimension))
+            ),
+            shared_table_map=immu.Map(
+                _raise_if_duplicate(_yield_children_nodes(node, XMLInlineTable))
+            ),
             default_locale=node.get("default_locale", "xx"),
             annotations=immu.Map(_yield_annotations(node)),
         )
@@ -85,15 +91,15 @@ class XMLAccessControl(models.AccessControl):
     def parse(cls, node: etree._Element):
         """Parse all <Access> XML node that are directly under this node."""
 
-        rules_gen = (
-            (role, _get_attr(item, "rule") == "allow")
-            for item in node.iterchildren(cls.tag)
-            for role in _get_attr(item, "roles").split(",")
-        )
-
         return cls(
-            public=node.get("public", "true") == "true",
-            rules=immu.Map(rules_gen),
+            public=_get_boolean(node, "public", True),
+            rules=immu.Map(
+                _raise_if_duplicate(
+                    (role, _get_attr(item, "rule") == "allow")
+                    for item in node.iterchildren(cls.tag)
+                    for role in _get_attr(item, "roles").split(",")
+                )
+            ),
         )
 
 
@@ -111,27 +117,33 @@ class XMLCube(models.Cube):
             raise MissingXMLNode(node.tag, name, "Table")
 
         dimension_map = OrderedDict(
-            _yield_children_nodes(node, XMLPrivateDimension, XMLDimensionUsage)
+            _raise_if_duplicate(
+                _yield_children_nodes(node, XMLPrivateDimension, XMLDimensionUsage)
+            )
         )
         if len(dimension_map) == 0:
             raise MissingXMLNode(node.tag, name, "Dimension")
 
         measure_map = OrderedDict(
-            _yield_children_nodes(node, XMLMeasure, XMLCalculatedMeasure)
+            _raise_if_duplicate(
+                _yield_children_nodes(node, XMLMeasure, XMLCalculatedMeasure)
+            )
         )
         if len(measure_map) == 0:
             raise MissingXMLNode(node.tag, name, "Measure")
 
         return cls(
             name=name,
-            captions=immu.Map(_yield_locale_pairs(node, "caption")),
+            captions=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "caption"))
+            ),
             acl=XMLAccessControl.parse(node),
             dimension_map=dimension_map,
             measure_map=measure_map,
             table=table,
             annotations=immu.Map(_yield_annotations(node)),
-            subset_table=node.get("subset_table", "false").lower() != "false",
-            visible=node.get("visible", "true").lower() != "false",
+            subset_table=_get_boolean(node, "subset_table", False),
+            visible=_get_boolean(node, "visible", True),
         )
 
 
@@ -149,10 +161,15 @@ class XMLDimensionUsage(models.DimensionUsage):
             source=_get_attr(node, "source"),
             foreign_key=_get_attr(node, "foreign_key"),
             annotations=immu.Map(_yield_annotations(node)),
-            captions=immu.Map(_yield_locale_pairs(node, "caption")),
-            hierarchy_map=OrderedDict(
-                _yield_children_nodes(node, XMLHierarchyUsage, attr="source")
+            captions=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "caption"))
             ),
+            hierarchy_map=OrderedDict(
+                _raise_if_duplicate(
+                    _yield_children_nodes(node, XMLHierarchyUsage, attr="source")
+                )
+            ),
+            visible=_get_boolean(node, "visible", True),
         )
 
 
@@ -169,10 +186,15 @@ class XMLHierarchyUsage(models.HierarchyUsage):
             name=name,
             source=_get_attr(node, "source"),
             annotations=immu.Map(_yield_annotations(node)),
-            captions=immu.Map(_yield_locale_pairs(node, "caption")),
-            level_map=OrderedDict(
-                _yield_children_nodes(node, XMLLevelUsage, attr="source")
+            captions=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "caption"))
             ),
+            level_map=OrderedDict(
+                _raise_if_duplicate(
+                    _yield_children_nodes(node, XMLLevelUsage, attr="source")
+                )
+            ),
+            visible=_get_boolean(node, "visible", True),
         )
 
 
@@ -189,10 +211,15 @@ class XMLLevelUsage(models.LevelUsage):
             name=name,
             source=_get_attr(node, "source"),
             annotations=immu.Map(_yield_annotations(node)),
-            captions=immu.Map(_yield_locale_pairs(node, "caption")),
-            property_map=OrderedDict(
-                _yield_children_nodes(node, XMLPropertyUsage, attr="source")
+            captions=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "caption"))
             ),
+            property_map=OrderedDict(
+                _raise_if_duplicate(
+                    _yield_children_nodes(node, XMLPropertyUsage, attr="source")
+                )
+            ),
+            visible=_get_boolean(node, "visible", True),
         )
 
 
@@ -209,7 +236,10 @@ class XMLPropertyUsage(models.PropertyUsage):
             name=name,
             source=_get_attr(node, "source"),
             annotations=immu.Map(_yield_annotations(node)),
-            captions=immu.Map(_yield_locale_pairs(node, "caption")),
+            captions=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "caption"))
+            ),
+            visible=_get_boolean(node, "visible", True),
         )
 
 
@@ -286,18 +316,20 @@ class XMLInlineTable(models.InlineTable):
         return headers, children[1:]
 
 
-class XMLSharedDimension(models.Dimension):
-    tag = "SharedDimension"
+class XMLDimension(models.Dimension):
+    tag = "Dimension"
 
     @classmethod
     def parse(cls, node: etree._Element, index: int):
-        """Parse a Shared <Dimension> XML node."""
+        """Parse a <Dimension> XML node."""
         name = _get_attr(node, "name")
         logger.debug("Parsing node <%s name='%s' />", cls.tag, name)
 
         dim_type = node.get("type")
 
-        hierarchy_map = OrderedDict(_yield_children_nodes(node, XMLHierarchy))
+        hierarchy_map = OrderedDict(
+            _raise_if_duplicate(_yield_children_nodes(node, XMLHierarchy))
+        )
 
         default_hierarchy = node.get("default_hierarchy")
         if default_hierarchy not in hierarchy_map:
@@ -305,22 +337,27 @@ class XMLSharedDimension(models.Dimension):
 
         return cls(
             name=name,
-            captions=immu.Map(_yield_locale_pairs(node, "caption")),
+            captions=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "caption"))
+            ),
             default_hierarchy=default_hierarchy,
             dim_type=DimensionType.from_str(dim_type),
             foreign_key=node.get("foreign_key"),
             hierarchy_map=hierarchy_map,
             annotations=immu.Map(_yield_annotations(node)),
+            visible=_get_boolean(node, "visible", True),
         )
 
 
-class XMLPrivateDimension(models.Dimension):
-    tag = "Dimension"
+class XMLSharedDimension(XMLDimension):
+    tag = "SharedDimension"
 
+
+class XMLPrivateDimension(XMLDimension):
     @classmethod
     def parse(cls, node: etree._Element, index: int):
         """Parse a Private <Dimension> XML node."""
-        dimension: cls = XMLSharedDimension.parse.__func__(cls, node, index)
+        dimension = super().parse(node, index)
 
         # foreign keys are required in Private Dimensions
         if dimension.foreign_key is None:
@@ -338,7 +375,9 @@ class XMLHierarchy(models.Hierarchy):
         name = _get_attr(node, "name")
         logger.debug("Parsing node <%s name='%s' />", cls.tag, name)
 
-        level_map = OrderedDict(_yield_children_nodes(node, XMLLevel))
+        level_map = OrderedDict(
+            _raise_if_duplicate(_yield_children_nodes(node, XMLLevel))
+        )
         if len(level_map) == 0:
             raise MissingXMLNode(node.tag, name, "Level")
 
@@ -349,11 +388,14 @@ class XMLHierarchy(models.Hierarchy):
         return cls(
             name=name,
             primary_key=node.get("primary_key", default_pk),
-            captions=immu.Map(_yield_locale_pairs(node, "caption")),
+            captions=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "caption"))
+            ),
             table=_find_table_ref(node),
             level_map=level_map,
             default_member=cls._parse_default_member(node),
             annotations=immu.Map(_yield_annotations(node)),
+            visible=_get_boolean(node, "visible", True),
         )
 
     @staticmethod
@@ -376,12 +418,20 @@ class XMLLevel(models.Level):
         return cls(
             name=name,
             depth=index + 1,
+            count=0,
             key_column=_get_attr(node, "key_column"),
             key_type=MemberType.from_str(key_type),
-            captions=immu.Map(_yield_locale_pairs(node, "caption")),
-            name_column_map=immu.Map(_yield_locale_pairs(node, "name_column")),
-            property_map=OrderedDict(_yield_children_nodes(node, XMLProperty)),
+            captions=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "caption"))
+            ),
+            name_column_map=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "name_column"))
+            ),
+            property_map=OrderedDict(
+                _raise_if_duplicate(_yield_children_nodes(node, XMLProperty))
+            ),
             annotations=immu.Map(_yield_annotations(node)),
+            visible=_get_boolean(node, "visible", True),
         )
 
 
@@ -396,16 +446,21 @@ class XMLProperty(models.Property):
 
         key_type = node.get("key_type")
 
-        keycol_map = immu.Map(_yield_locale_pairs(node, "key_column"))
+        keycol_map = immu.Map(
+            _raise_if_duplicate(_yield_locale_pairs(node, "key_column"))
+        )
         if len(keycol_map) == 0:
             raise MissingXMLAttribute(node.tag, "key_column")
 
         return cls(
             name=name,
             annotations=immu.Map(_yield_annotations(node)),
-            captions=immu.Map(_yield_locale_pairs(node, "caption")),
+            captions=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "caption"))
+            ),
             key_column_map=keycol_map,
             key_type=MemberType.from_str(key_type),
+            visible=_get_boolean(node, "visible", True),
         )
 
 
@@ -423,8 +478,11 @@ class XMLMeasure(models.Measure):
             key_column=_get_attr(node, "key_column"),
             aggregator=cls._get_aggregator(node),
             annotations=immu.Map(_yield_annotations(node)),
-            captions=immu.Map(_yield_locale_pairs(node, "caption")),
-            submeasures=immu.Map(_yield_children_nodes(node, cls)),
+            captions=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "caption"))
+            ),
+            submeasures=immu.Map(_raise_if_duplicate(_yield_children_nodes(node, cls))),
+            visible=_get_boolean(node, "visible", True),
         )
 
     @staticmethod
@@ -443,7 +501,9 @@ class XMLMeasure(models.Measure):
 
         # if there's an <Aggregation> node, get its `type`
         # else get the `<Measure>`'s `aggregator` attribute
-        node, attr = (mea_node, "aggregator") if agg_node is None else (agg_node, "type")
+        node, attr = (
+            (mea_node, "aggregator") if agg_node is None else (agg_node, "type")
+        )
         value = _get_attr(node, attr)
 
         try:
@@ -473,12 +533,17 @@ class XMLCalculatedMeasure(models.CalculatedMeasure):
             name=name,
             formula=cls._parse_formula(_get_attr(node, "formula")),
             annotations=immu.Map(_yield_annotations(node)),
-            captions=immu.Map(_yield_locale_pairs(node, "caption")),
+            captions=immu.Map(
+                _raise_if_duplicate(_yield_locale_pairs(node, "caption"))
+            ),
+            visible=_get_boolean(node, "visible", True),
         )
 
 
 def _find_table_ref(node: etree._Element):
-    gen_tables = (item for item in node.iterchildren("InlineTable", "Table", "TableUsage"))
+    gen_tables = (
+        item for item in node.iterchildren("InlineTable", "Table", "TableUsage")
+    )
     table_node = next(gen_tables, None)
 
     if table_node is None:
@@ -506,11 +571,27 @@ def _get_attr(node: etree._Element, attr: str) -> str:
         return str(value)
 
 
+def _get_boolean(node: etree._Element, attr: str, default: bool) -> bool:
+    value = node.get(attr)
+    return default if value is None else (value.lower() in TRUTHY_STRINGS)
+
+
+def _raise_if_duplicate(
+    generator: Iterable[Tuple[str, T]],
+    exc_cls: Type[DuplicateKeyError] = DuplicateKeyError,
+):
+    seen_keys = set()
+    for key, value in generator:
+        if key in seen_keys:
+            raise exc_cls(key)
+        seen_keys.add(key)
+        yield key, value
+
+
 def _yield_annotations(node: etree._Element) -> Iterable[Tuple[str, Optional[str]]]:
     """Yields a pair of (name, value) for each Annotation in the node."""
-    return (
-        (_get_attr(item, "name"), item.text)
-        for item in node.iterchildren("Annotation")
+    return _raise_if_duplicate(
+        (_get_attr(item, "name"), item.text) for item in node.iterchildren("Annotation")
     )
 
 
@@ -526,8 +607,9 @@ def _yield_children_nodes(
         yield _get_attr(item, attr), reducer(item, index)
 
 
-def _yield_locale_pairs(node: etree._Element,
-                        attribute: str) -> Generator[Tuple[str, str], None, None]:
+def _yield_locale_pairs(
+    node: etree._Element, attribute: str
+) -> Generator[Tuple[str, str], None, None]:
     attr_value = node.get(attribute)
     if attr_value is not None:
         yield ("xx", attr_value)
@@ -550,9 +632,9 @@ def parse_xml_schema(source: Union[str, Path, TextIO]) -> XMLSchema:
     - A local path (as a :class:`pathlib.Path`) to a XML file
     - A not-binary read-only :class:`TextIO` instance for a file-like object
     """
-    parser = etree.XMLParser(encoding="utf-8",
-                             remove_blank_text=True,
-                             remove_comments=True)
+    parser = etree.XMLParser(
+        encoding="utf-8", remove_blank_text=True, remove_comments=True
+    )
 
     # if argument is str, is assumed to be a raw XML string
     if isinstance(source, str):
