@@ -1,6 +1,7 @@
 import logging
 from pathlib import Path
 from typing import Union
+from urllib.parse import parse_qs
 
 from typing_extensions import deprecated
 
@@ -8,6 +9,7 @@ from tesseract_olap.backend import Backend, CacheProvider, DummyProvider, LfuPro
 from tesseract_olap.exceptions.query import InvalidQuery
 from tesseract_olap.exceptions.server import UnknownBackendError
 from tesseract_olap.query import (
+    AnyQuery,
     AnyRequest,
     DataQuery,
     DataRequest,
@@ -18,7 +20,7 @@ from tesseract_olap.schema import Schema, SchemaTraverser
 
 from .schema import setup_schema
 
-logger = logging.getLogger("tesseract_olap.server")
+logger = logging.getLogger(__name__)
 
 
 class OlapServer:
@@ -56,13 +58,25 @@ class OlapServer:
         return self.schema.schema
 
     def build_query(self, request: AnyRequest):
+        """Uses the internal schema to validate a Request and generate its
+        matching Query instance."""
         if isinstance(request, DataRequest):
             return DataQuery.from_request(self.schema, request)
-        elif isinstance(request, MembersRequest):
+        if isinstance(request, MembersRequest):
             return MembersQuery.from_request(self.schema, request)
-        else:
-            msg = "Attempt to build a Query without using a valid Request instance."
-            raise InvalidQuery(msg)
+
+        raise InvalidQuery(
+            "Attempt to build a Query using an invalid Request instance."
+        )
+
+    def clear_cache(self) -> None:
+        """Clears all stored items in the cache."""
+        self.cache.clear()
+
+    def generate_sql(self, query: AnyQuery) -> str:
+        """Builds the SQL query for a Query instance, using the backend currently
+        configured in this server."""
+        return self.backend.generate_sql(query)
 
     @deprecated(
         "The session() method allows to reuse a connection for multiple queries."
@@ -82,8 +96,8 @@ class OlapServer:
         except Exception:
             return False
 
-    def session(self):
-        return self.backend.new_session(cache=self.cache)
+    def session(self, **kwargs):
+        return self.backend.new_session(cache=self.cache, **kwargs)
 
     def validate(self):
         """Verifies the information declared in the Schema matches the data
@@ -106,15 +120,16 @@ def _setup_backend(dsn: str):
 
 def _setup_cache(dsn: str) -> CacheProvider:
     """Generates a new instance of a CacheProvider bundled in this package."""
-    if dsn == ":memory:":
-        return LfuProvider()
+    if dsn.startswith(":memory:"):
+        try:
+            params = parse_qs(dsn.replace(":memory:", ""), strict_parsing=True)
+            maxsize = params.get("maxsize", ["64"])
+            dfsize = params.get("dfsize", ["150"])
+            return LfuProvider(maxsize=int(maxsize[0]), dfsize=int(dfsize[0]))
+        except ValueError:
+            return LfuProvider()
 
-    if (
-        dsn.startswith("valkey:")
-        or dsn.startswith("valkeys:")
-        or dsn.startswith("redis:")
-        or dsn.startswith("rediss:")
-    ):
+    if dsn.startswith(("valkey:", "valkeys:", "redis:", "rediss:")):
         from tesseract_olap.backend.valkey import ValkeyProvider
 
         return ValkeyProvider(dsn)
